@@ -20,11 +20,6 @@ from ai.h2o.sparkling.ml.algos.H2ODeepLearning import H2ODeepLearning
 from ai.h2o.sparkling.ml.algos.H2OXGBoost import H2OXGBoost
 from ai.h2o.sparkling.ml.algos.H2OGBM import H2OGBM
 from ai.h2o.sparkling.ml.algos.H2OGLM import H2OGLM
-from ai.h2o.sparkling.ml.algos.H2OGAM import H2OGAM
 from ai.h2o.sparkling.ml.algos.H2OGridSearch import H2OGridSearch
 from ai.h2o.sparkling.ml.algos.H2OAutoML import H2OAutoML
 from ai.h2o.sparkling.ml.algos.H2ODRF import H2ODRF
-from ai.h2o.sparkling.ml.algos.H2OCoxPH import H2OCoxPH
-from ai.h2o.sparkling.ml.algos.H2OIsolationForest import H2OIsolationForest
-from ai.h2o.sparkling.ml.algos.H2ORuleFit import H2ORuleFit
-from ai.h2o.sparkling.ml.algos.H2OStackedEnsemble import H2OStackedEnsemble
