@@ -16,8 +16,4 @@
 #
 
 from ai.h2o.sparkling.ml.features.H2OTargetEncoder import H2OTargetEncoder
-from ai.h2o.sparkling.ml.features.H2OAutoEncoder import H2OAutoEncoder
 from ai.h2o.sparkling.ml.features.ColumnPruner import ColumnPruner
-from ai.h2o.sparkling.ml.features.H2OWord2Vec import H2OWord2Vec
-from ai.h2o.sparkling.ml.features.H2OPCA import H2OPCA
-from ai.h2o.sparkling.ml.features.H2OGLRM import H2OGLRM
