@@ -38,13 +38,25 @@ have multiple connections to separate H2O servers open at the same time. Such fa
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from distutils.version import StrictVersion
-import sys
-
 
 from .cluster import H2OCluster
 from .server import H2OLocalServer
 from .connection import H2OConnection
 from .connection import H2OConnectionConf
 
+import colorama
+from distutils.version import StrictVersion
+import sys
+
+if not hasattr(colorama, '__version__'):
+    print("[WARNING] H2O requires colorama module of version 0.3.8 or newer. You have older version .\n"
+          "You can upgrade to the newest version of the module running from the command line\n"
+          "    $ pip%s install --upgrade colorama" % (sys.version_info[0]))
+    sys.exit(1)
+elif StrictVersion(colorama.__version__) < StrictVersion("0.3.8"):
+    print("[WARNING] H2O requires colorama module of version 0.3.8 or newer. You have version %s.\n"
+          "You can upgrade to the newest version of the module running from the command line\n"
+          "    $ pip%s install --upgrade colorama" % (colorama.__version__, sys.version_info[0]))
+    sys.exit(1)
+    
 __all__ = ("H2OCluster", "H2OConnection", "H2OLocalServer", "H2OConnectionConf")
