@@ -13,7 +13,6 @@ Class for communication with an H2O server.
 :license:   Apache License Version 2.0 (see LICENSE for details)
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
-from h2o.utils.compatibility import *  # NOQA
 
 import atexit
 import os
@@ -33,6 +32,7 @@ from h2o.exceptions import H2OConnectionError, H2OServerError, H2OResponseError,
 from h2o.schemas.error import H2OErrorV3, H2OModelBuilderErrorV3
 from h2o.two_dim_table import H2OTwoDimTable
 from h2o.utils.metaclass import CallableString, BackwardsCompatible, h2o_meta
+from h2o.utils.compatibility import *  # NOQA
 from h2o.utils.shared_utils import stringify_list, stringify_dict, print2
 from h2o.utils.typechecks import (assert_is_type, assert_matches, assert_satisfies, is_type, numeric)
 from h2o.model.metrics_base import (H2ORegressionModelMetrics, H2OClusteringModelMetrics, H2OBinomialModelMetrics,
@@ -805,9 +805,6 @@ class H2OConnection(h2o_meta()):
         content_type = response.headers.get("Content-Type", "")
         if ";" in content_type:  # Remove a ";charset=..." part
             content_type = content_type[:content_type.index(";")]
-            
-        # this is needed so that response.text() works correctly
-        response.encoding = response.headers.get("Character-Encoding", response.encoding)
 
         # Auto-detect response type by its content-type. Decode JSON, all other responses pass as-is.
         if content_type == "application/json":
